@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ using KeePass.UI;
 using KeePass.Util;
 using KeePass.Util.XmlSerialization;
 
+using KeePassLib;
 using KeePassLib.Delegates;
 using KeePassLib.Native;
 using KeePassLib.Serialization;
@@ -39,15 +40,24 @@ using KeePassLib.Utility;
 
 namespace KeePass.App.Configuration
 {
-	[XmlType(TypeName = AppConfigEx.StrXmlTypeName)]
+	[XmlType(AppConfigEx.StrXmlTypeName)]
 	public sealed class AppConfigEx
 	{
-		private const string StrXmlTypeName = "Configuration";
+		internal const string StrXmlTypeName = "Configuration";
+
+		private string m_strSearchString = null;
 
 		public AppConfigEx()
 		{
 		}
 
+#if DEBUG
+		~AppConfigEx()
+		{
+			Debug.Assert(m_strSearchString == null);
+		}
+#endif
+
 		private AceMeta m_meta = null;
 		public AceMeta Meta
 		{
@@ -168,6 +178,21 @@ namespace KeePass.App.Configuration
 			}
 		}
 
+		private AceSearch m_aceSearch = null;
+		public AceSearch Search
+		{
+			get
+			{
+				if(m_aceSearch == null) m_aceSearch = new AceSearch();
+				return m_aceSearch;
+			}
+			set
+			{
+				if(value == null) throw new ArgumentNullException("value");
+				m_aceSearch = value;
+			}
+		}
+
 		private AceDefaults m_def = null;
 		public AceDefaults Defaults
 		{
@@ -238,6 +263,7 @@ namespace KeePass.App.Configuration
 		{
 			AceMeta aceMeta = this.Meta; // m_meta might be null
 			AceApplication aceApp = this.Application; // m_aceApp might be null
+			AceSearch aceSearch = this.Search; // m_aceSearch might be null
 			AceDefaults aceDef = this.Defaults; // m_def might be null
 
 			aceMeta.OmitItemsWithDefaultValues = true;
@@ -254,7 +280,9 @@ namespace KeePass.App.Configuration
 
 			aceApp.TriggerSystem = Program.TriggerSystem;
 
-			SearchUtil.PrepareForSerialize(aceDef.SearchParameters);
+			SearchUtil.PrepareForSerialize(aceSearch.LastUsedProfile);
+			foreach(SearchParameters sp in aceSearch.UserProfiles)
+				SearchUtil.PrepareForSerialize(sp);
 
 			const int m = 64; // Maximum number of compatibility items
 			List<string> l = aceApp.PluginCompatibility;
@@ -264,6 +292,7 @@ namespace KeePass.App.Configuration
 		internal void OnLoad()
 		{
 			AceMainWindow aceMW = this.MainWindow; // m_uiMainWindow might be null
+			AceSearch aceSearch = this.Search; // m_aceSearch might be null
 			AceDefaults aceDef = this.Defaults; // m_def might be null
 
 			// aceInt.UrlSchemeOverrides.SetDefaultsIfEmpty();
@@ -272,17 +301,20 @@ namespace KeePass.App.Configuration
 			ChangePathsRelAbs(true);
 
 			// Remove invalid columns
-			List<AceColumn> vColumns = aceMW.EntryListColumns;
+			List<AceColumn> lColumns = aceMW.EntryListColumns;
 			int i = 0;
-			while(i < vColumns.Count)
+			while(i < lColumns.Count)
 			{
-				if(((int)vColumns[i].Type < 0) || ((int)vColumns[i].Type >=
+				if(((int)lColumns[i].Type < 0) || ((int)lColumns[i].Type >=
 					(int)AceColumnType.Count))
-					vColumns.RemoveAt(i);
+					lColumns.RemoveAt(i);
 				else ++i;
 			}
 
-			SearchUtil.FinishDeserialize(aceDef.SearchParameters);
+			SearchUtil.FinishDeserialize(aceSearch.LastUsedProfile);
+			foreach(SearchParameters sp in aceSearch.UserProfiles)
+				SearchUtil.FinishDeserialize(sp);
+
 			DpiScale();
 
 			if(aceMW.EscMinimizesToTray) // For backward compatibility
@@ -330,10 +362,12 @@ namespace KeePass.App.Configuration
 
 			ChangePathsRelAbs(false);
 			ObfuscateCred(true);
+			RemoveSensitiveInfo(true);
 		}
 
 		internal void OnSavePost()
 		{
+			RemoveSensitiveInfo(false);
 			ObfuscateCred(false);
 			ChangePathsRelAbs(true);
 		}
@@ -409,6 +443,27 @@ namespace KeePass.App.Configuration
 			else aceInt.ProxyPassword = StrUtil.Deobfuscate(aceInt.ProxyPassword);
 		}
 
+		private void RemoveSensitiveInfo(bool bRemove)
+		{
+			SearchParameters sp = this.Search.LastUsedProfile;
+
+			if(bRemove)
+			{
+				Debug.Assert(m_strSearchString == null);
+				m_strSearchString = sp.SearchString;
+				sp.SearchString = string.Empty;
+			}
+			else
+			{
+				if(m_strSearchString != null)
+				{
+					sp.SearchString = m_strSearchString;
+					m_strSearchString = null;
+				}
+				else { Debug.Assert(false); }
+			}
+		}
+
 		private void DpiScale()
 		{
 			AceMeta aceMeta = this.Meta; // m_meta might be null
@@ -583,6 +638,8 @@ namespace KeePass.App.Configuration
 				case "/Configuration/PasswordGenerator/AutoGeneratedPasswordsProfile":
 				case "/Configuration/PasswordGenerator/LastUsedProfile":
 				case "/Configuration/PasswordGenerator/UserProfiles/Profile":
+				case "/Configuration/Search/LastUsedProfile":
+				case "/Configuration/Search/UserProfiles/Profile":
 					o.ContentMode = XmContentMode.Replace;
 					break;
 
@@ -654,6 +711,7 @@ namespace KeePass.App.Configuration
 					break;
 
 				case "/Configuration/PasswordGenerator/UserProfiles/Profile":
+				case "/Configuration/Search/UserProfiles/Profile":
 					strA = XmlUtil.SafeInnerXml(xn, "Name");
 					break;
 
