@@ -1,6 +1,6 @@
 ﻿/*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -610,14 +610,14 @@ namespace KeePass.Forms
 
 		protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
 		{
-			if(keyData == Keys.Escape)
+			bool bDown;
+			if(!NativeMethods.GetKeyMessageState(ref msg, out bDown))
+				return base.ProcessCmdKey(ref msg, keyData);
+
+			if(keyData == Keys.Escape) // No modifiers
 			{
-				bool? obKeyDown = NativeMethods.IsKeyDownMessage(ref msg);
-				if(obKeyDown.HasValue)
-				{
-					if(obKeyDown.Value) this.Close();
-					return true;
-				}
+				if(bDown) Close();
+				return true;
 			}
 
 			return base.ProcessCmdKey(ref msg, keyData);
