@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -387,7 +387,7 @@ namespace KeePass.Forms
 			Debug.Assert(p.ToString() == strPropertyName);
 
 			ListViewItem lvi = m_cdxPolicy.CreateItem(Program.Config.Security.Policy,
-				strPropertyName, null, AppPolicy.GetName(p) + "*");
+				strPropertyName, null, AppPolicy.GetName(p) + " *");
 			lvi.SubItems.Add(AppPolicy.GetDesc(p));
 		}
 
@@ -422,6 +422,8 @@ namespace KeePass.Forms
 
 			bool? obNoMin = null;
 			if(MonoWorkarounds.IsRequired(1418)) obNoMin = true;
+			bool? obNoFocus = null;
+			if(MonoWorkarounds.IsRequired(1976)) obNoFocus = true;
 
 			m_lvGuiOptions.Columns.Add(KPRes.Options); // Resize below
 
@@ -506,9 +508,9 @@ namespace KeePass.Forms
 			m_cdxGuiOptions.CreateItem(Program.Config.MainWindow, "FocusResultsAfterQuickFind",
 				lvg, KPRes.FocusResultsAfterQuickSearch);
 			m_cdxGuiOptions.CreateItem(Program.Config.MainWindow, "FocusQuickFindOnRestore",
-				lvg, KPRes.FocusQuickFindOnRestore);
+				lvg, KPRes.FocusQuickFindOnRestore, obNoFocus);
 			m_cdxGuiOptions.CreateItem(Program.Config.MainWindow, "FocusQuickFindOnUntray",
-				lvg, KPRes.FocusQuickFindOnUntray);
+				lvg, KPRes.FocusQuickFindOnUntray, obNoFocus);
 
 			lvg = new ListViewGroup(KPRes.TrayIcon);
 			m_lvGuiOptions.Groups.Add(lvg);
@@ -635,6 +637,8 @@ namespace KeePass.Forms
 				lvg, KPRes.AutoTypeMatchByUrlHostInTitle);
 			m_cdxAdvanced.CreateItem(Program.Config.Integration, "AutoTypeMatchByTagInTitle",
 				lvg, KPRes.AutoTypeMatchByTagInTitle);
+			m_cdxAdvanced.CreateItem(Program.Config.Integration, "AutoTypeMatchNormDashes",
+				lvg, KPRes.ConsiderDashesEq + " (-, \u2010, \u2011, \u2012, \u2013, \u2014, \u2015, \u2212)");
 			m_cdxAdvanced.CreateItem(Program.Config.Integration, "AutoTypeExpiredCanMatch",
 				lvg, KPRes.ExpiredEntriesCanMatch);
 			m_cdxAdvanced.CreateItem(Program.Config.Integration, "AutoTypeAlwaysShowSelDialog",
@@ -945,7 +949,7 @@ namespace KeePass.Forms
 
 		private void OnBtnFileExtCreate(object sender, EventArgs e)
 		{
-			// ShellUtil.RegisterExtension(AppDefs.FileExtension.FileExt, AppDefs.FileExtension.ExtId,
+			// ShellUtil.RegisterExtension(AppDefs.FileExtension.FileExt, AppDefs.FileExtension.FileExtId,
 			//	KPRes.FileExtName2, WinUtil.GetExecutable(), PwDefs.ShortProductName, true);
 			WinUtil.RunElevated(WinUtil.GetExecutable(), "-" +
 				AppDefs.CommandLineOptions.FileExtRegister, false);
@@ -954,7 +958,7 @@ namespace KeePass.Forms
 		private void OnBtnFileExtRemove(object sender, EventArgs e)
 		{
 			// ShellUtil.UnregisterExtension(AppDefs.FileExtension.FileExt,
-			//	AppDefs.FileExtension.ExtId);
+			//	AppDefs.FileExtension.FileExtId);
 			WinUtil.RunElevated(WinUtil.GetExecutable(), "-" +
 				AppDefs.CommandLineOptions.FileExtUnregister, false);
 		}
@@ -1052,5 +1056,21 @@ namespace KeePass.Forms
 				UpdateButtonImages();
 			}
 		}
+
+		private void OnBtnHelpSource(object sender, EventArgs e)
+		{
+			HelpSourceForm hsf = new HelpSourceForm();
+			UIUtil.ShowDialogAndDestroy(hsf);
+		}
+
+		private void OnSecOptExLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
+		{
+			AppHelp.ShowHelp(AppDefs.HelpTopics.Security, AppDefs.HelpTopics.SecurityOptEx);
+		}
+
+		private void OnSecOptAdmLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
+		{
+			AppHelp.ShowHelp(AppDefs.HelpTopics.Security, AppDefs.HelpTopics.SecurityOptAdm);
+		}
 	}
 }
