@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -204,6 +204,26 @@ namespace KeePassLib.Native
 			return m_tDesktop.Value;
 		}
 
+		private static bool? g_obWayland = null;
+		internal static bool IsWayland()
+		{
+			if(!g_obWayland.HasValue)
+			{
+				bool b = false;
+				try
+				{
+					// https://www.freedesktop.org/software/systemd/man/pam_systemd.html
+					b = ((Environment.GetEnvironmentVariable("XDG_SESSION_TYPE") ??
+						string.Empty).Trim().Equals("wayland", StrUtil.CaseIgnoreCmp));
+				}
+				catch(Exception) { Debug.Assert(false); }
+
+				g_obWayland = b;
+			}
+
+			return g_obWayland.Value;
+		}
+
 #if (!KeePassLibSD && !KeePassUAP)
 		public static string RunConsoleApp(string strAppPath, string strParams)
 		{
@@ -583,6 +603,7 @@ namespace KeePassLib.Native
 			ProcessStartInfo psi = new ProcessStartInfo();
 			if(!string.IsNullOrEmpty(strFile)) psi.FileName = strFile;
 			if(!string.IsNullOrEmpty(strArgs)) psi.Arguments = strArgs;
+			psi.UseShellExecute = true;
 
 			StartProcess(psi);
 		}
@@ -601,47 +622,71 @@ namespace KeePassLib.Native
 
 			string strFileOrg = psi.FileName;
 			if(string.IsNullOrEmpty(strFileOrg)) { Debug.Assert(false); return null; }
+			string strArgsOrg = psi.Arguments;
 
 			Process p;
 			try
 			{
-				string strFile = strFileOrg;
-
-				string[] vUrlEncSchemes = new string[] {
-					"file:", "ftp:", "ftps:", "http:", "https:",
-					"mailto:", "scp:", "sftp:"
-				};
-				foreach(string strPfx in vUrlEncSchemes)
-				{
-					if(strFile.StartsWith(strPfx, StrUtil.CaseIgnoreCmp))
-					{
-						Debug.Assert(string.IsNullOrEmpty(psi.Arguments));
-
-						strFile = strFile.Replace("\"", "%22");
-						strFile = strFile.Replace("\'", "%27");
-						strFile = strFile.Replace("\\", "%5C");
-						break;
-					}
-				}
-
-				if(IsUnix())
-				{
-					// Mono's Process.Start method replaces '\\' by '/',
-					// which may cause a different file to be executed;
-					// therefore, we refuse to start such files
-					if(strFile.Contains("\\") && MonoWorkarounds.IsRequired(190417))
-						throw new ArgumentException(KLRes.PathBackslash);
-
-					strFile = strFile.Replace("\\", "\\\\"); // If WA not required
-					strFile = strFile.Replace("\"", "\\\"");
-				}
-
-				psi.FileName = strFile;
+				CustomizeProcessStartInfo(psi);
 				p = Process.Start(psi);
 			}
-			finally { psi.FileName = strFileOrg; }
+			finally
+			{
+				psi.FileName = strFileOrg; // Restore
+				psi.Arguments = strArgsOrg;
+			}
 
 			return p;
 		}
+
+		private static void CustomizeProcessStartInfo(ProcessStartInfo psi)
+		{
+			string strFile = psi.FileName, strArgs = psi.Arguments;
+
+			string[] vUrlEncSchemes = new string[] {
+				"file:", "ftp:", "ftps:", "http:", "https:",
+				"mailto:", "scp:", "sftp:"
+			};
+			foreach(string strPfx in vUrlEncSchemes)
+			{
+				if(strFile.StartsWith(strPfx, StrUtil.CaseIgnoreCmp))
+				{
+					Debug.Assert(string.IsNullOrEmpty(strArgs));
+
+					strFile = strFile.Replace("\"", "%22");
+					strFile = strFile.Replace("\'", "%27");
+					strFile = strFile.Replace("\\", "%5C");
+					break;
+				}
+			}
+
+			if(IsUnix())
+			{
+				if(MonoWorkarounds.IsRequired(19836) && string.IsNullOrEmpty(strArgs))
+				{
+					if(Regex.IsMatch(strFile, "^[a-zA-Z][a-zA-Z0-9\\+\\-\\.]*:",
+						RegexOptions.Singleline) ||
+						strFile.EndsWith(".html", StrUtil.CaseIgnoreCmp))
+					{
+						bool bMacOSX = (GetPlatformID() == PlatformID.MacOSX);
+
+						strArgs = "\"" + EncodeDataToArgs(strFile) + "\"";
+						strFile = (bMacOSX ? "open" : "xdg-open");
+					}
+				}
+
+				// Mono's Process.Start method replaces '\\' by '/',
+				// which may cause a different file to be executed;
+				// therefore, we refuse to start such files
+				if(strFile.Contains("\\") && MonoWorkarounds.IsRequired(190417))
+					throw new ArgumentException(KLRes.PathBackslash);
+
+				strFile = strFile.Replace("\\", "\\\\"); // If WA not required
+				strFile = strFile.Replace("\"", "\\\"");
+			}
+
+			psi.FileName = strFile;
+			psi.Arguments = strArgs;
+		}
 	}
 }
