@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
 using System;
 using System.Diagnostics;
 using System.Drawing;
-using System.Drawing.Drawing2D;
-using System.Drawing.Imaging;
 using System.IO;
 using System.Runtime.InteropServices;
 using System.Security;
@@ -117,7 +115,7 @@ namespace KeePass.Native
 			return IntPtr.Zero;
 		}
 
-		private static readonly char[] m_vWindowTrim = { '\r', '\n' };
+		private static readonly char[] g_vWindowNL = new char[] { '\r', '\n' };
 		internal static void GetForegroundWindowInfo(out IntPtr hWnd,
 			out string strWindowText, bool bTrimWindow)
 		{
@@ -131,7 +129,7 @@ namespace KeePass.Native
 				if(!string.IsNullOrEmpty(strWindowText))
 				{
 					if(bTrimWindow) strWindowText = strWindowText.Trim();
-					else strWindowText = strWindowText.Trim(m_vWindowTrim);
+					else strWindowText = strWindowText.Trim(g_vWindowNL);
 				}
 			}
 		}
@@ -439,7 +437,7 @@ namespace KeePass.Native
 			return null;
 		}
 
-		internal static bool SHGetFileInfo(string strPath, int nPrefImgDim,
+		internal static bool SHGetFileInfo(string strPath, int dxImg, int dyImg,
 			out Image img, out string strDisplayName)
 		{
 			img = null;
@@ -457,14 +455,7 @@ namespace KeePass.Native
 				{
 					using(Icon ico = Icon.FromHandle(fi.hIcon)) // Doesn't take ownership
 					{
-						img = new Bitmap(nPrefImgDim, nPrefImgDim);
-						using(Graphics g = Graphics.FromImage(img))
-						{
-							g.Clear(Color.Transparent);
-							g.InterpolationMode = InterpolationMode.HighQualityBicubic;
-							g.SmoothingMode = SmoothingMode.HighQuality;
-							g.DrawIcon(ico, new Rectangle(0, 0, nPrefImgDim, nPrefImgDim));
-						}
+						img = UIUtil.IconToBitmap(ico, dxImg, dyImg);
 					}
 
 					if(!DestroyIcon(fi.hIcon)) { Debug.Assert(false); }
@@ -736,7 +727,7 @@ namespace KeePass.Native
 			return false;
 		}
 
-		internal static bool? IsKeyDownMessage(ref Message m)
+		private static bool? IsKeyDownMessage(ref Message m)
 		{
 			if(m.Msg == NativeMethods.WM_KEYDOWN) return true;
 			if(m.Msg == NativeMethods.WM_KEYUP) return false;
@@ -745,6 +736,20 @@ namespace KeePass.Native
 			return null;
 		}
 
+		internal static bool GetKeyMessageState(ref Message m, out bool bDown)
+		{
+			bool? obKeyDown = IsKeyDownMessage(ref m);
+			if(!obKeyDown.HasValue)
+			{
+				Debug.Assert(false);
+				bDown = false;
+				return false;
+			}
+
+			bDown = obKeyDown.Value;
+			return true;
+		}
+
 		/* internal static string GetKeyboardLayoutNameEx()
 		{
 			StringBuilder sb = new StringBuilder(KL_NAMELENGTH + 1);
