@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -103,7 +103,6 @@ namespace KeePass.Util
 			{
 				// SendKeys is not used anymore, thus the following is
 				// not required:
-
 				// // Enable new SendInput method; see
 				// // https://msdn.microsoft.com/en-us/library/system.windows.forms.sendkeys.aspx
 				// ConfigurationManager.AppSettings.Set("SendKeys", "SendInput");
@@ -111,42 +110,44 @@ namespace KeePass.Util
 			catch(Exception) { Debug.Assert(false); }
 		}
 
-		internal static bool MatchWindows(string strFilter, string strWindow)
+		internal static bool IsMatchWindow(string strWindow, string strFilter)
 		{
-			Debug.Assert(strFilter != null); if(strFilter == null) return false;
-			Debug.Assert(strWindow != null); if(strWindow == null) return false;
+			if(strWindow == null) { Debug.Assert(false); return false; }
+			if(strFilter == null) { Debug.Assert(false); return false; }
 
-			string strF = strFilter.Trim();
+			Debug.Assert(NormalizeWindowText(strWindow) == strWindow); // Should be done by caller
+			string strF = NormalizeWindowText(strFilter);
 
-			/* bool bArbStart = strF.StartsWith("*"), bArbEnd = strF.EndsWith("*");
-
-			if(bArbStart) strF = strF.Remove(0, 1);
-			if(bArbEnd) strF = strF.Substring(0, strF.Length - 1);
-
-			if(bArbStart && bArbEnd)
-				return (strWindow.IndexOf(strF, StrUtil.CaseIgnoreCmp) >= 0);
-			else if(bArbStart)
-				return strWindow.EndsWith(strF, StrUtil.CaseIgnoreCmp);
-			else if(bArbEnd)
-				return strWindow.StartsWith(strF, StrUtil.CaseIgnoreCmp);
-
-			return strWindow.Equals(strF, StrUtil.CaseIgnoreCmp); */
-
-			if(strF.StartsWith(@"//") && strF.EndsWith(@"//") && (strF.Length > 4))
+			int ccF = strF.Length;
+			if((ccF > 4) && (strF[0] == '/') && (strF[1] == '/') &&
+				(strF[ccF - 2] == '/') && (strF[ccF - 1] == '/'))
 			{
 				try
 				{
-					Regex rx = new Regex(strF.Substring(2, strF.Length - 4),
-						RegexOptions.IgnoreCase);
-
-					return rx.IsMatch(strWindow);					
+					string strRx = strF.Substring(2, ccF - 4);
+					return Regex.IsMatch(strWindow, strRx, RegexOptions.IgnoreCase);
 				}
-				catch(Exception) { }
+				catch(Exception) { return false; }
 			}
 
 			return StrUtil.SimplePatternMatch(strF, strWindow, StrUtil.CaseIgnoreCmp);
 		}
 
+		private static bool IsMatchSub(string strWindow, string strSub)
+		{
+			if(strWindow == null) { Debug.Assert(false); return false; }
+			if(strSub == null) { Debug.Assert(false); return false; }
+
+			Debug.Assert(NormalizeWindowText(strWindow) == strWindow); // Should be done by caller
+			string strS = NormalizeWindowText(strSub);
+
+			// If strS is empty, return false, because strS is a substring
+			// that must occur, not a filter
+			if(strS.Length == 0) return false;
+
+			return (strWindow.IndexOf(strS, StrUtil.CaseIgnoreCmp) >= 0);
+		}
+
 		private static bool Execute(AutoTypeCtx ctx)
 		{
 			if(ctx == null) { Debug.Assert(false); return false; }
@@ -298,12 +299,8 @@ namespace KeePass.Util
 			// in order to allow selecting the first item as default one
 			foreach(AutoTypeAssociation a in pwe.AutoType.Associations)
 			{
-				string strWndSpec = a.WindowName;
-				if(strWndSpec == null) { Debug.Assert(false); continue; }
-
-				strWndSpec = SprEngine.Compile(strWndSpec.Trim(), sprCtx);
-
-				if(MatchWindows(strWndSpec, strWindow))
+				string strFilter = SprEngine.Compile(a.WindowName, sprCtx);
+				if(IsMatchWindow(strWindow, strFilter))
 				{
 					string strSeq = a.Sequence;
 					if(string.IsNullOrEmpty(strSeq))
@@ -318,19 +315,17 @@ namespace KeePass.Util
 			if(Program.Config.Integration.AutoTypeMatchByTitle)
 			{
 				string strTitle = SprEngine.Compile(pwe.Strings.ReadSafe(
-					PwDefs.TitleField).Trim(), sprCtx);
-				if((strTitle.Length > 0) && (strWindow.IndexOf(strTitle,
-					StrUtil.CaseIgnoreCmp) >= 0))
+					PwDefs.TitleField), sprCtx);
+				if(IsMatchSub(strWindow, strTitle))
 					AddSequence(l, pwe.GetAutoTypeSequence());
 			}
 
-			string strCmpUrl = null; // To cache compiled URL
+			string strCmpUrl = null; // To cache the compiled URL
 			if(Program.Config.Integration.AutoTypeMatchByUrlInTitle)
 			{
 				strCmpUrl = SprEngine.Compile(pwe.Strings.ReadSafe(
-					PwDefs.UrlField).Trim(), sprCtx);
-				if((strCmpUrl.Length > 0) && (strWindow.IndexOf(strCmpUrl,
-					StrUtil.CaseIgnoreCmp) >= 0))
+					PwDefs.UrlField), sprCtx);
+				if(IsMatchSub(strWindow, strCmpUrl))
 					AddSequence(l, pwe.GetAutoTypeSequence());
 			}
 
@@ -338,9 +333,9 @@ namespace KeePass.Util
 			{
 				if(strCmpUrl == null)
 					strCmpUrl = SprEngine.Compile(pwe.Strings.ReadSafe(
-						PwDefs.UrlField).Trim(), sprCtx);
+						PwDefs.UrlField), sprCtx);
 
-				string strCleanUrl = StrUtil.RemovePlaceholders(strCmpUrl);
+				string strCleanUrl = StrUtil.RemovePlaceholders(strCmpUrl).Trim();
 				string strHost = UrlUtil.GetHost(strCleanUrl);
 
 				if(strHost.StartsWith("www.", StrUtil.CaseIgnoreCmp) &&
@@ -348,8 +343,7 @@ namespace KeePass.Util
 					strCleanUrl.StartsWith("https:", StrUtil.CaseIgnoreCmp)))
 					strHost = strHost.Substring(4);
 
-				if((strHost.Length > 0) && (strWindow.IndexOf(strHost,
-					StrUtil.CaseIgnoreCmp) >= 0))
+				if(IsMatchSub(strWindow, strHost))
 					AddSequence(l, pwe.GetAutoTypeSequence());
 			}
 
@@ -357,9 +351,7 @@ namespace KeePass.Util
 			{
 				foreach(string strTag in pwe.Tags)
 				{
-					if(string.IsNullOrEmpty(strTag)) { Debug.Assert(false); continue; }
-
-					if(strWindow.IndexOf(strTag, StrUtil.CaseIgnoreCmp) >= 0)
+					if(IsMatchSub(strWindow, strTag))
 					{
 						AddSequence(l, pwe.GetAutoTypeSequence());
 						break;
@@ -465,13 +457,7 @@ namespace KeePass.Util
 
 			IntPtr hWnd;
 			string strWindow;
-			try
-			{
-				// hWnd = NativeMethods.GetForegroundWindowHandle();
-				// strWindow = NativeMethods.GetWindowText(hWnd);
-				NativeMethods.GetForegroundWindowInfo(out hWnd, out strWindow, true);
-			}
-			catch(Exception) { Debug.Assert(false); hWnd = IntPtr.Zero; strWindow = null; }
+			GetForegroundWindowInfo(out hWnd, out strWindow);
 
 			// if(string.IsNullOrEmpty(strWindow)) return false;
 			if(strWindow == null) { Debug.Assert(false); return false; }
@@ -618,11 +604,7 @@ namespace KeePass.Util
 
 			IntPtr hWnd;
 			string strWindow;
-			try
-			{
-				NativeMethods.GetForegroundWindowInfo(out hWnd, out strWindow, true);
-			}
-			catch(Exception) { hWnd = IntPtr.Zero; strWindow = null; }
+			GetForegroundWindowInfo(out hWnd, out strWindow);
 
 			if(!NativeLib.IsUnix())
 			{
@@ -688,5 +670,48 @@ namespace KeePass.Util
 
 			return null;
 		} */
+
+		private static readonly char[] g_vNormToHyphen = new char[] {
+			// Sync with UI option name
+			'\u2010', // Hyphen
+			'\u2011', // Non-breaking hyphen
+			'\u2012', // Figure dash
+			'\u2013', // En dash
+			'\u2014', // Em dash
+			'\u2015', // Horizontal bar
+			'\u2212' // Minus sign
+		};
+		internal static string NormalizeWindowText(string str)
+		{
+			if(string.IsNullOrEmpty(str)) return string.Empty;
+
+			str = str.Trim();
+
+			if(Program.Config.Integration.AutoTypeMatchNormDashes &&
+				(str.IndexOfAny(g_vNormToHyphen) >= 0))
+			{
+				for(int i = 0; i < g_vNormToHyphen.Length; ++i)
+					str = str.Replace(g_vNormToHyphen[i], '-');
+			}
+
+			return str;
+		}
+
+		private static void GetForegroundWindowInfo(out IntPtr hWnd, out string strWindow)
+		{
+			try
+			{
+				NativeMethods.GetForegroundWindowInfo(out hWnd, out strWindow, false);
+			}
+			catch(Exception)
+			{
+				Debug.Assert(false);
+				hWnd = IntPtr.Zero;
+				strWindow = null;
+				return;
+			}
+
+			strWindow = NormalizeWindowText(strWindow);
+		}
 	}
 }
